@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,12 +31,8 @@
 #include <iostream>
 #include <set>
 
-#include <fcntl.h>
-#ifdef WIN32
-#include <io.h>
-#else
-#include <unistd.h>
-#endif
+#include "safefcntl.h"
+#include "safeunistd.h"
 
 #include "omega.h"
 #include "utils.h"
@@ -45,7 +41,7 @@
 
 using namespace std;
 
-static const char * DEFAULT_STEM_LANGUAGE = "english";
+static const char DEFAULT_STEM_LANGUAGE[] = "english";
 
 Xapian::Enquire * enquire;
 Xapian::Database db;
@@ -72,8 +68,11 @@ string query_string;
 int threshold = 0;
 
 bool sort_numeric = false;
-Xapian::valueno sort_key = Xapian::valueno(-1);
+Xapian::valueno sort_key = Xapian::BAD_VALUENO; // Don't sort.
 bool sort_ascending = true;
+bool sort_after = false;
+Xapian::Enquire::docid_order docid_order = Xapian::Enquire::ASCENDING;
+
 Xapian::valueno collapse_key = 0;
 bool collapse = false;
 
@@ -191,12 +190,12 @@ try {
 	    Xapian::PostingIterator p = db.postlist_begin(v);
 	    if (p != db.postlist_end(v)) docid = *p;
 	}
-	
+
 	if (docid != 0) {
 	    Xapian::RSet tmprset;
 	    tmprset.add_document(docid);
 
-	    ExpandDeciderOmega decider(db);
+	    OmegaExpandDecider decider(db);
 	    Xapian::ESet eset(enquire->get_eset(40, tmprset, &decider));
 	    for (Xapian::ESetIterator i = eset.begin(); i != eset.end(); i++) {
 		if ((*i).empty()) continue;
@@ -266,24 +265,10 @@ try {
 
     // date range filters
     val = cgi_params.find("START");
-    // DATE1 is the deprecated name - check for backward compatibility
-    if (val == cgi_params.end()) val = cgi_params.find("DATE1");
     if (val != cgi_params.end()) date_start = val->second;
     val = cgi_params.find("END");
-    // DATE2 is the deprecated name - check for backward compatibility
-    if (val == cgi_params.end()) val = cgi_params.find("DATE2");
     if (val != cgi_params.end()) date_end = val->second;
     val = cgi_params.find("SPAN");
-    // DAYSMINUS is the deprecated name - check for backward compatibility
-    if (val == cgi_params.end()) {
-	val = cgi_params.find("DAYSMINUS");
-	if (val != cgi_params.end()) {
-	    // Range used to be DAYSMINUS days before DATE1
-	    // Now it's SPAN days after START or before END
-	    date_end = date_start;
-	    date_start = "";
-	}
-    }
     if (val != cgi_params.end()) date_span = val->second;
 
     filters += date_start + filter_sep + date_end + filter_sep + date_span
@@ -304,7 +289,24 @@ try {
 	if (!v.empty()) {
 	    collapse_key = atoi(v.c_str());
 	    collapse = true;
-	    filters += filter_sep + v;
+	    filters += filter_sep + int_to_string(collapse_key);
+	}
+    }
+
+    // docid order
+    val = cgi_params.find("DOCIDORDER");
+    if (val != cgi_params.end()) {
+	const string & v = val->second;
+	if (!v.empty()) {
+	    char ch = v[0];
+	    if (ch == 'D') {
+		docid_order = Xapian::Enquire::DESCENDING;
+		filters += 'D';
+	    } else if (ch != 'A') {
+		docid_order = Xapian::Enquire::DONT_CARE;
+	    } else {
+		filters += 'X';
+	    }
 	}
     }
 
@@ -323,16 +325,28 @@ try {
 	if (val != cgi_params.end()) {
 	    sort_ascending = (atoi(val->second.c_str()) == 0);
 	}
-	// FIXME: add SORT and SORTREVERSE to filters too!  But in a compatible
-	// way ideally...
+	val = cgi_params.find("SORTAFTER");
+	if (val != cgi_params.end()) {
+	    sort_after = (atoi(val->second.c_str()) != 0);
+	}
+	// Add the sorting related options to filters too.
+	filters += int_to_string(sort_key);
+	if (sort_after) {
+	    if (sort_ascending) {
+		filters += 'F';
+	    } else {
+		filters += 'R';
+	    }
+	} else {
+	    if (!sort_ascending) {
+		filters += 'r';
+	    }
+	}
     }
 
     // min_hits (fill mset past topdoc+(hits_per_page+1) to
     // topdoc+max(hits_per_page+1,min_hits)
     val = cgi_params.find("MINHITS");
-    // In Omega <= 0.6.3, MINHITS was MIN_HITS - renamed to be consistent
-    // with the naming of other CGI parameters.
-    if (val == cgi_params.end()) val = cgi_params.find("MIN_HITS");
     if (val != cgi_params.end()) {
 	min_hits = atol(val->second.c_str());
     }
