@@ -48,6 +48,7 @@
 #include "md5wrap.h"
 #include "metaxmlparse.h"
 #include "myhtmlparse.h"
+#include "runfilter.h"
 #include "sample.h"
 #include "stringutils.h"
 #include "utf8convert.h"
@@ -57,24 +58,6 @@
 
 #include "gnu_getopt.h"
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
-
-#ifdef __WIN32__
-# ifndef WIFEXITED
-#  define WIFEXITED(status) (status != -1)
-# endif
-# ifndef WEXITSTATUS
-#  define WEXITSTATUS(status) (status)
-# endif
-#endif
-
-#ifdef _MSC_VER
-# define popen _popen
-# define pclose _pclose
-#endif
-
 #ifndef HAVE_MKDTEMP
 extern char * mkdtemp(char *);
 #endif
@@ -145,9 +128,6 @@ static bool ensure_tmpdir() {
     return (p != NULL);
 }
 
-struct ReadError {};
-struct NoSuchFilter {};
-
 static string
 file_to_string(const string &file)
 {
@@ -156,32 +136,9 @@ file_to_string(const string &file)
     return out;
 }
 
-static string
-stdout_to_string(const string &cmd)
+static void
+get_pdf_metainfo(const string & safefile, string &title, string &keywords)
 {
-    string out;
-    FILE * fh = popen(cmd.c_str(), "r");
-    if (fh == NULL) throw ReadError();
-    while (!feof(fh)) {
-	char buf[4096];
-	size_t len = fread(buf, 1, 4096, fh);
-	if (ferror(fh)) {
-	    (void)pclose(fh);
-	    throw ReadError();
-	}
-	out.append(buf, len);
-    }
-    int status = pclose(fh);
-    if (status != 0) {
-	if (WIFEXITED(status) && WEXITSTATUS(status) == 127) {
-	    throw NoSuchFilter();
-	}
-	throw ReadError();
-    }
-    return out;
-}
-
-static void get_pdf_metainfo(const string & safefile, string &title, string &keywords) {
     try {
 	string pdfinfo = stdout_to_string("pdfinfo -enc UTF-8 " + safefile);
 
@@ -289,8 +246,8 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	}
 	get_pdf_metainfo(safefile, title, keywords);
     } else if (mimetype == "application/postscript") {
-	// There simply doesn't seem to be a Unicode capabable PostScript to
-	// text convertor (e.g. pstotext always outputs ISO-8859-1).  The only
+	// There simply doesn't seem to be a Unicode capable PostScript to
+	// text converter (e.g. pstotext always outputs ISO-8859-1).  The only
 	// solution seems to be to convert via PDF using ps2pdf and then
 	// pdftotext.  This gives plausible looking UTF-8 output for some
 	// Chinese PostScript files I found using Google.  It also has the
@@ -392,6 +349,27 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    cout << "\"" << cmd << "\" failed - skipping\n";
 	    return;
 	}
+    } else if (mimetype == "application/x-abiword") {
+	// FIXME: Implement support for metadata.
+	try {
+	    XmlParser xmlparser;
+	    xmlparser.parse_html(file_to_string(file));
+	    dump = xmlparser.dump;
+	} catch (ReadError) {
+	    cout << "can't read \"" << file << "\" - skipping\n";
+	    return;
+	}
+    } else if (mimetype == "application/x-abiword-compressed") {
+	// FIXME: Implement support for metadata.
+	string cmd = "gzip -dc " + shell_protect(file);
+	try {
+	    XmlParser xmlparser;
+	    xmlparser.parse_html(stdout_to_string(cmd));
+	    dump = xmlparser.dump;
+	} catch (ReadError) {
+	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    return;
+	}
     } else if (mimetype == "text/rtf") {
 	// The --text option unhelpfully converts all non-ASCII characters to
 	// "?" so we use --html instead, which write HTML entities.
@@ -425,6 +403,20 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    cout << "\"" << cmd << "\" failed - skipping\n";
 	    return;
 	}
+    } else if (mimetype == "application/x-dvi") {
+	// FIXME: -e2 means "UTF-8", but that results in "fi", "ff", "ffi", etc
+	// appearing as single ligatures.  For European languages, it's
+	// actually better to use -e2 (ISO-8859-1) and then convert, so let's
+	// do that for now until we handle Unicode "compatibility
+	// decompositions".
+	string cmd = "catdvi -e2 -s " + shell_protect(file);
+	try {
+	    dump = stdout_to_string(cmd);
+	    convert_to_utf8(dump, "ISO-8859-1");
+	} catch (ReadError) {
+	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    return;
+	}
     } else {
 	// Don't know how to index this type.
 	cout << "unknown MIME type - skipping\n";
@@ -590,6 +582,8 @@ index_directory(size_t depth_limit, const string &dir,
 			cout << "Filter for \"" << mimetype << "\" not installed - ignoring extension \"" << ext << "\"" << endl;
 			mime_map.erase(mt);
 		    }
+		} else {
+		    cout << "Unknown extension: \"" << file << "\" - skipping" << endl;
 		}
 		continue;
 	    }
@@ -681,6 +675,8 @@ main(int argc, char **argv)
     mime_map["wpd"] = "application/vnd.wordperfect";
     mime_map["wps"] = "application/vnd.ms-works";
     mime_map["wpt"] = "application/vnd.ms-works"; // Works template
+    mime_map["abw"] = "application/x-abiword"; // AbiWord
+    mime_map["zabw"] = "application/x-abiword-compressed"; // AbiWord compressed
     mime_map["rtf"] = "text/rtf";
     // Other MS formats:
     mime_map["xls"] = "application/vnd.ms-excel";
@@ -692,6 +688,8 @@ main(int argc, char **argv)
     mime_map["pl"] = "text/x-perl";
     mime_map["pm"] = "text/x-perl";
     mime_map["pod"] = "text/x-perl";
+    // Other formats:
+    mime_map["dvi"] = "application/x-dvi";
 
     while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:lp", longopts, NULL))!=EOF) {
 	switch (getopt_ret) {
