@@ -220,9 +220,7 @@ set_probabilistic(const string &oldp)
     qp.set_default_op(default_op);
     qp.set_database(db);
     // std::map::insert() won't overwrite an existing entry, so we'll prefer
-    // probabilistic prefixes to boolean ones in the unlikely event that both
-    // exist for the same term prefix.  We'll also prefer the first specified
-    // prefix.
+    // the first user_prefix for which a particular term prefix is specified.
     map<string, string>::const_iterator pfx = option.lower_bound("prefix,");
     for (; pfx != option.end() && startswith(pfx->first, "prefix,"); ++pfx) {
 	string user_prefix = pfx->first.substr(7);
@@ -766,6 +764,7 @@ CMD_mod,
 CMD_msize,
 CMD_msizeexact,
 CMD_mul,
+CMD_muldiv,
 CMD_ne,
 CMD_nice,
 CMD_not,
@@ -815,9 +814,6 @@ struct func_attrib {
     char ensure;
 };
 
-#define STRINGIZE(N) _STRINGIZE(N)
-#define _STRINGIZE(N) #N
-
 #define T(F,A,B,C,D) {STRINGIZE(F),{CMD_##F,A,B,C,D}}
 struct func_desc {
     const char *name;
@@ -884,6 +880,7 @@ T(mod,		   2, 2, N, 0), // integer modulus
 T(msize,	   0, 0, N, M), // number of matches
 T(msizeexact,	   0, 0, N, M), // is $msize exact?
 T(mul,		   2, N, N, 0), // multiply a list of numbers
+T(muldiv,	   3, 3, N, 0), // calculate A*B/C
 T(ne,		   2, 2, N, 0), // test not equal
 T(nice,		   1, 1, N, 0), // pretty print integer (with thousands sep)
 T(not,		   1, 1, N, 0), // logical not
@@ -1181,18 +1178,20 @@ eval(const string &fmt, const vector<string> &param)
 		    format = "%d byte";
 		} else if (size < 1024) {
 		    format = "%d bytes";
-		} else if (size < 1024*1024) {
+		} else {
+		    if (size < 1024*1024) {
+			format = "%d.%cK";
+		    } else {
+			size /= 1024;
+			if (size < 1024*1024) {
+			    format = "%d.%cM";
+			} else {
+			    size /= 1024;
+			    format = "%d.%cG";
+			}
+		    }
 		    intpart = size / 1024;
 		    fraction = size % 1024;
-		    format = "%d.%cK";
-		} else if (size < 1024*1024*1024) {
-		    intpart = size / (1024 * 1024);
-		    fraction = size % (1024 * 1024);
-		    format = "%d.%cM";
-		} else {
-		    intpart = size / (1024 * 1024 * 1024);
-		    fraction = size % (1024 * 1024 * 1024);
-		    format = "%d.%cG";
 		}
 		if (format) {
 		    char buf[200];
@@ -1524,6 +1523,16 @@ eval(const string &fmt, const vector<string> &param)
 		value = int_to_string(total);
 		break;
 	    }
+	    case CMD_muldiv: {
+		int denom = string_to_int(args[2]);
+		if (denom == 0) {
+		    value = "divide by 0";
+		} else {
+		    int num = string_to_int(args[0]) * string_to_int(args[1]);
+		    value = int_to_string(num / denom);
+		}
+		break;
+	    }
             case CMD_ne:
 		if (args[0] != args[1]) value = "true";
 		break;
